import 'dart:math';

import 'package:flutter/material.dart';

enum PageTransitionType {
  fadeIn,
  transferRight,
  transferUp,
  slideInLeft,
  slideInRight,
  slideInUp,
  slideInDown,
  slideLeft,
  slideRight,
  slideUp,
  slideDown,
  slideParallaxLeft,
  slideParallaxRight,
  slideParallaxUp,
  slideParallaxDown,
  slideZoomLeft,
  slideZoomRight,
  slideZoomUp,
  slideZoomDown,
  rippleRightUp,
  rippleLeftUp,
  rippleLeftDown,
  rippleRightDown,
  rippleMiddle,
}

class _RippleClipper extends CustomClipper<Path> {
  _RippleClipper(this.origin, this.progress);

  final String origin;
  final double progress;

  @override
  Path getClip(Size size) {
    Map center = <String, Offset>{
      'Left': Offset(0, size.height),
      'Right': Offset(size.width, size.height),
      'LeftDown': const Offset(0, 0),
      'RightDown': Offset(size.width, 0),
      'Middle': Offset(size.width * .5, size.height * .5),
    };
    Path path = Path();
    double radius = sqrt(pow(size.width, 2) + pow(size.height, 2));
    path.addOval(
        Rect.fromCircle(center: center[origin], radius: radius * progress));
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) => true;
}

class GPageTransition {
  final PageTransitionType _pageTransitionType;
  final Curve _curve = Curves.linear;
  final Animation<double> _animation, _secondaryAnimation;
  final Widget _child;

  final Tween<Offset> _t1 = Tween(
    begin: const Offset(1.0, 0.0),
    end: const Offset(0.0, 0.0),
  );

  final Tween<Offset> _t2 = Tween(
    begin: const Offset(-1.0, 0.0),
    end: const Offset(0.0, 0.0),
  );

  final Tween<Offset> _t3 = Tween(
    begin: const Offset(0.0, 1.0),
    end: const Offset(0.0, 0.0),
  );

  final Tween<Offset> _t4 = Tween(
    begin: const Offset(0.0, -1.0),
    end: const Offset(0.0, 0.0),
  );

  final Tween<Offset> _t5 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(-1.0, 0.0),
  );

  final Tween<Offset> _t6 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(1.0, 0.0),
  );

  final Tween<Offset> _t7 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(0.0, -1.0),
  );

  final Tween<Offset> _t8 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(0.0, 1.0),
  );

  final Tween<Offset> _t9 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(-0.8, 0.0),
  );

  final Tween<Offset> _t10 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(0.8, 0.0),
  );

  final Tween<Offset> _t11 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(0.0, -0.8),
  );

  final Tween<Offset> _t12 = Tween(
    begin: const Offset(0.0, 0.0),
    end: const Offset(0.0, 0.8),
  );

  final Tween<double> _t13 = Tween(
    begin: 1.0,
    end: 0.9,
  );

  final Tween<double> _t14 = Tween(
    begin: 0.9,
    end: 1.0,
  );

  GPageTransition(
    this._pageTransitionType,
    this._animation,
    this._secondaryAnimation,
    this._child,
  );

  Widget getEffect() {
    switch (_pageTransitionType) {
      case PageTransitionType.fadeIn:
        return _getFadeIn();

      case PageTransitionType.transferRight:
        return _getTransfer(_t1, _t5, _t14, _t13);

      case PageTransitionType.transferUp:
        return _getTransfer(_t3, _t7, _t14, _t13);

      case PageTransitionType.slideInLeft:
        return _getSlideIn(_t1);

      case PageTransitionType.slideInRight:
        return _getSlideIn(_t2);

      case PageTransitionType.slideInUp:
        return _getSlideIn(_t3);

      case PageTransitionType.slideInDown:
        return _getSlideIn(_t4);

      case PageTransitionType.slideLeft:
        return _getSlide(_t1, _t5);

      case PageTransitionType.slideRight:
        return _getSlide(_t2, _t6);

      case PageTransitionType.slideUp:
        return _getSlide(_t3, _t7);

      case PageTransitionType.slideDown:
        return _getSlide(_t4, _t8);

      case PageTransitionType.slideParallaxLeft:
        return _getSlide(_t1, _t9);

      case PageTransitionType.slideParallaxRight:
        return _getSlide(_t2, _t10);

      case PageTransitionType.slideParallaxUp:
        return _getSlide(_t3, _t11);

      case PageTransitionType.slideParallaxDown:
        return _getSlide(_t4, _t12);

      case PageTransitionType.slideZoomLeft:
        return _getZoomSlide(_t1, _t13);

      case PageTransitionType.slideZoomRight:
        return _getZoomSlide(_t2, _t13);

      case PageTransitionType.slideZoomUp:
        return _getZoomSlide(_t3, _t13);

      case PageTransitionType.slideZoomDown:
        return _getZoomSlide(_t4, _t13);

      case PageTransitionType.rippleRightUp:
        return _getRipple('Right');

      case PageTransitionType.rippleLeftUp:
        return _getRipple('Left');

      case PageTransitionType.rippleLeftDown:
        return _getRipple('LeftDown');

      case PageTransitionType.rippleRightDown:
        return _getRipple('RightDown');

      case PageTransitionType.rippleMiddle:
        return _getRipple('Middle');

      default:
        return _getFadeIn();
    }
  }

  Widget _getFadeIn() => FadeTransition(opacity: _animation, child: _child);

  Widget _getTransfer(
          Tween<Offset> animationTween,
          Tween<Offset> secondaryAnimationTween,
          Tween<double> animationScaleTween,
          Tween<double> secondaryAnimationScaleTween) =>
      SlideTransition(
        position: animationTween.animate(
          CurvedAnimation(
            parent: _animation,
            curve: Interval(0.3, 0.7, curve: _curve),
          ),
        ),
        child: ScaleTransition(
          scale: animationScaleTween.animate(
            CurvedAnimation(
              parent: _animation,
              curve: Interval(0.0, 0.3, curve: _curve),
            ),
          ),
          child: ScaleTransition(
            scale: animationScaleTween.animate(
              CurvedAnimation(
                parent: _animation,
                curve: Interval(0.7, 1.0, curve: _curve),
              ),
            ),
            child: SlideTransition(
              position: secondaryAnimationTween.animate(
                CurvedAnimation(
                  parent: _secondaryAnimation,
                  curve: Interval(0.3, 0.7, curve: _curve),
                ),
              ),
              child: ScaleTransition(
                scale: secondaryAnimationScaleTween.animate(
                  CurvedAnimation(
                    parent: _secondaryAnimation,
                    curve: Interval(0.0, 0.3, curve: _curve),
                  ),
                ),
                child: ScaleTransition(
                  scale: secondaryAnimationScaleTween.animate(
                    CurvedAnimation(
                      parent: _secondaryAnimation,
                      curve: Interval(0.7, 1.0, curve: _curve),
                    ),
                  ),
                  child: _child,
                ),
              ),
            ),
          ),
        ),
      );

  Widget _getSlideIn(Tween<Offset> tween) => SlideTransition(
        position: tween.animate(
          CurvedAnimation(parent: _animation, curve: _curve),
        ),
        child: _child,
      );

  Widget _getSlide(Tween<Offset> animationTween,
          Tween<Offset> secondaryAnimationTween) =>
      SlideTransition(
        position: animationTween.animate(_animation),
        child: SlideTransition(
          position: secondaryAnimationTween.animate(_secondaryAnimation),
          child: _child,
        ),
      );

  Widget _getZoomSlide(Tween<Offset> animationTween,
          Tween<double> secondaryAnimationTween) =>
      SlideTransition(
        position: animationTween.animate(_animation),
        child: ScaleTransition(
          scale: secondaryAnimationTween.animate(_secondaryAnimation),
          child: _child,
        ),
      );

  Widget _getRipple(String origin) => ClipPath(
        clipper: _RippleClipper(origin, _animation.value),
        child: _child,
      );
}
